#pragma once

#include <types/vertex.h>
#include <types/texture.h>
#include <types/shader.h>
#include <types/entity/entity.h>
#include <J3ML/LinearAlgebra/Vector4.h>

using J3ML::LinearAlgebra::Vector4;
using J3ML::LinearAlgebra::Vector2;

// TODO: Move data to Entity / or rename to DataModelEntry
struct ByteArray
{
    std::vector<uint8_t> bytes;
};

class Serializable
{
    virtual ByteArray Serialize() const;
    virtual Serializable Deserialize(const ByteArray& data);
};


// A wrapper around a Tree Hierarchy Data Model.
class DataModel : public Entity {
public:

    Entity * GetParent() const override
    {
        return nullptr;
    }

    std::vector<Entity> GetFlatEntityList();

    template <class T>
    void Serialize(T& archive)
    {
        Entity::SerializeMemberData(archive);
        archive & GetFlatEntityList();
    }

    void SetParent(Entity *parent) override
    {
        throw std::runtime_error("Cannot set parent of Hierarchy Root!");
    }
    DataModel() : Entity() {

    }
    [[nodiscard]] int getEntityCount() const;
    [[nodiscard]] int getMobyCount() const;
};

class World : public DataModel {
private:
    Vector4 globalLightColor = {0, 0, 0, 0};
    Vector4 globalFogColor = {0, 0, 0, 0};
    Vector2 globalFogRange = {0, 0};
    GLenum globalFogMode = NULL; //There's Linear, GL_EXP, GL_EXP2.
    GLfloat globalFogDensity = 0.0f;

public:
    World() : DataModel() {}
    void setAmbientLightColor(GLfloat red, GLfloat green, GLfloat blue, GLfloat alpha);
    void setGlobalFogColor(GLfloat red, GLfloat green, GLfloat blue, GLfloat alpha);
    void setGlobalFogRange(GLfloat start, GLfloat end);
    void setGlobalFogDensity(GLfloat density);
    void setGlobalFogMode(GLenum mode);

    GLenum getGlobalFogMode();
    Vector4 getGlobalFogColor();
    Vector2 getGlobalFogRange();
    Vector4 getAmbientLightColor();
    GLfloat getGlobalFogDensity();

    std::string name;
    std::vector<VertexArray> geometryList;
    std::vector<Texture*> textureList;
    std::vector<Shader> shaderList;

};
