#include <thread>
#include <engine/engine.h>
#include <cube.h>
#include <sphere.h>
#include <freeCam.h>
#include <demoSky.h>

int main(int argc, char** argv)
{
    engine->window = new ReWindow::RWindow("Re3D Test Application", 1152, 864, RenderingAPI::OPENGL);
    engine->world = new(World);
    engine->init();
    engine->window->setVsyncEnabled(false);
    engine->window->setResizable(false);
    engine->world->setAmbientLightColor(1.0f, 1.0f, 1.0f, 0.0f);
    Shader test("test", engine->workingDir + "/assets/shaders/defaultVertex.glsl", engine->workingDir + "/assets/shaders/defaultFragment.glsl");

    auto* camera = new(FreeCam);
    camera->SetPos({0.0f, -2.0f, -5.0f});
    camera->angle.y = 0.0f;
    camera->SetParent(engine->world);

    auto* cube1 = new(Cube);
    cube1->name = "cube1";
    cube1->SetPos({-5, -2, 0});
    cube1->velAngle = cube1->lAngle();
    cube1->SetParent(engine->world);

    auto* cube2 = new(Sphere2);
    cube2->name = "sphere";
    cube2->SetPos({5, -2, 0});
    cube2->velAngle = cube2->rAngle();
    cube2->SetParent(engine->world);

    auto *skybox = new(DemoSkybox);
    skybox->angle = {0,0,0};
    skybox->SetParent(engine->world);

    engine->renderLoop();
}


#ifdef WINDOWS_SMH
extern "C" {
    int wmain(int argc, wchar_t* argv[])
    {
        return main(0, nullptr);
    }
}
#endif